@@ -26,7 +26,7 @@
 #include <stdlib.h>
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include "eel-gconf-extensions.h"
 #include "rb-dialog.h"
@@ -62,7 +62,7 @@ eel_gconf_handle_error (GError **error)
 	g_return_val_if_fail (error != NULL, FALSE);
 
 	if (*error != NULL) {
-		g_warning ((*error)->message);
+		g_warning ("%s", (*error)->message);
 		g_error_free (*error);
 		*error = NULL;
 
