@@ -7,6 +7,14 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
+# The Rhythmbox authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and Rhythmbox. This permission is above and beyond the permissions granted
+# by the GPL license by which Rhythmbox is covered. If you modify this code
+# you may extend this exception to your version of the code, but you are not
+# obligated to do so. If you do not wish to do so, delete this exception
+# statement from your version.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -234,17 +242,79 @@ gobject.type_register (FadingImage)
 
 
 class ArtDisplayWidget (FadingImage):
+	__gsignals__ = {
+			'pixbuf-dropped' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (rhythmdb.Entry, gtk.gdk.Pixbuf)),
+			'uri-dropped' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (rhythmdb.Entry, gobject.TYPE_STRING))
+			}
+
 	def __init__ (self, missing_image):
 		super (ArtDisplayWidget, self).__init__ (missing_image)
 		self.set_padding (0, 5)
+		self.ddg_id = self.connect ('drag-data-get', self.drag_data_get)
+		self.ddr_id = self.connect ('drag-data-received', self.drag_data_received)
 		self.current_entry, self.working = None, False
 		self.current_pixbuf, self.current_uri = None, None
 
+	def disconnect_handlers (self):
+ 		super (ArtDisplayWidget, self).disconnect_handlers ()
+		self.disconnect (self.ddg_id)
+		self.disconnect (self.ddr_id)
+	
+	def update_dnd_targets (self):
+		targets = None
+		if self.current_entry:
+			targets = gtk.target_list_add_image_targets (targets)
+			targets = gtk.target_list_add_uri_targets (targets)
+			targets = gtk.target_list_add_text_targets (targets)
+		if targets:
+			self.drag_dest_set (gtk.DEST_DEFAULT_ALL, targets, gtk.gdk.ACTION_COPY)
+		else:
+			self.drag_dest_unset ()
+
+		targets = None
+		if self.current_pixbuf:
+			targets = gtk.target_list_add_image_targets (targets, writable=True)
+		if self.current_uri:
+			targets = gtk.target_list_add_uri_targets (targets)
+		if targets:
+			self.drag_source_set (gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
+		else:
+			self.drag_source_unset ()
+
+	def update_tooltips (self, working):
+		if not self.current_entry:
+			self.set_tooltip_text (None)
+		elif working:
+			self.set_tooltip_text (_("Searching... drop artwork here"))
+		else:
+			self.set_tooltip_text (_("Drop artwork here"))
+
 	def set (self, entry, pixbuf, uri, working):
 		self.current_entry = entry
 		self.current_pixbuf = pixbuf
 		self.current_uri = uri
 		self.set_current_art (pixbuf, working)
+		self.update_dnd_targets ()
+		self.update_tooltips (working)
+
+	def drag_data_get (self, widget, drag_context, selection_data, info, timestamp):
+		if self.current_pixbuf:
+			selection_data.set_pixbuf (self.current_pixbuf)
+		if self.current_uri:
+			selection_data.set_uris ([self.current_uri])
+
+	def drag_data_received (self, widget, drag_context, x, y, selection_data, info, timestamp):
+		entry = self.current_entry
+		pixbuf = selection_data.get_pixbuf ()
+		uris = selection_data.get_uris ()
+		text = selection_data.get_text ()
+		if pixbuf:
+			self.emit ('pixbuf-dropped', entry, pixbuf)
+		elif uris:
+			self.emit ('uri-dropped', entry, uris[0])
+		elif text:
+			self.emit ('uri-dropped', entry, text)
+gobject.type_register (ArtDisplayWidget)
 
 
 class ArtDisplayPlugin (rb.Plugin):
@@ -254,28 +324,42 @@ class ArtDisplayPlugin (rb.Plugin):
 	def activate (self, shell):
 		self.shell = shell
 		sp = shell.get_player ()
-		self.pec_id = sp.connect ('playing-song-changed', self.playing_entry_changed)
-		self.pc_id = sp.connect ('playing-changed', self.playing_changed)
+		self.player_cb_ids = (
+			sp.connect ('playing-song-changed', self.playing_entry_changed),
+			sp.connect ('playing-changed', self.playing_changed)
+		)
 		db = shell.get_property ("db")
-		self.eemr_art_id = db.connect_after ('entry-extra-metadata-request::rb:coverArt', self.cover_art_request)
-		self.eemn_art_id = db.connect_after ('entry-extra-metadata-notify::rb:coverArt', self.cover_art_notify)
-		self.eemn_art_uri_id = db.connect_after ('entry-extra-metadata-notify::rb:coverArt-uri', self.cover_art_uri_notify)
+		self.db_cb_ids = (
+			db.connect_after ('entry-extra-metadata-request::rb:coverArt', self.cover_art_request),
+			db.connect_after ('entry-extra-metadata-notify::rb:coverArt', self.cover_art_notify),
+			db.connect_after ('entry-extra-metadata-request::rb:coverArt-uri', self.cover_art_uri_request),
+			db.connect_after ('entry-extra-metadata-notify::rb:coverArt-uri', self.cover_art_uri_notify),
+			db.connect_after ('entry-extra-metadata-gather', self.cover_art_uri_gather),
+		)
 		self.art_widget = ArtDisplayWidget (self.find_file (ART_MISSING_ICON + ".svg"))
-		shell.add_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
+		self.art_widget.connect ('pixbuf-dropped', self.on_set_pixbuf)
+		self.art_widget.connect ('uri-dropped', self.on_set_uri)
+		self.art_container = gtk.VBox ()
+		self.art_container.pack_start (self.art_widget, padding=6)
+		shell.add_widget (self.art_container, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_db = CoverArtDatabase ()
 		self.current_entry, self.current_pixbuf = None, None
 		self.playing_entry_changed (sp, sp.get_playing_entry ())
 
 	def deactivate (self, shell):
 		self.shell = None
+
 		sp = shell.get_player ()
-		sp.disconnect (self.pec_id)
-		sp.disconnect (self.pc_id)
+		for id in self.player_cb_ids:
+			sp.disconnect (id)
+		self.player_cb_ids = ()
+
 		db = shell.get_property ("db")
-		db.disconnect (self.eemr_art_id)
-		db.disconnect (self.eemn_art_id)
-		db.disconnect (self.eemn_art_uri_id)
-		shell.remove_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
+		for id in self.db_cb_ids:
+			db.disconnect (id)
+		self.db_cb_ids = ()
+
+		shell.remove_widget (self.art_container, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_widget.disconnect_handlers ()
 		self.art_widget = None
 		self.art_db = None
@@ -292,7 +376,7 @@ class ArtDisplayPlugin (rb.Plugin):
 		db = self.shell.get_property ("db")
 
 		self.art_widget.set (entry, None, None, True)
-		self.art_widget.show ()
+		self.art_container.show_all ()
 		# Intitates search in the database (which checks art cache, internet etc.)
 		self.current_entry = entry
 		self.current_pixbuf = None
@@ -351,5 +435,21 @@ class ArtDisplayPlugin (rb.Plugin):
 					pass
 
 		print "got cover art URI notification: %s" % (uri)
-		rb.Loader().get_url (uri, loader_cb)
+		l = rb.Loader()
+		l.get_url (uri, loader_cb)
 
+	def cover_art_uri_request (self, db, entry):
+		if entry == self.current_entry:
+			return self.art_widget.current_uri
+
+	def cover_art_uri_gather (self, db, entry, metadata):
+		if entry == self.current_entry and self.art_widget.current_uri:
+			metadata ['rb:coverArt-uri'] = self.art_widget.current_uri
+
+	def on_set_pixbuf (self, widget, entry, pixbuf):
+		db = self.shell.get_property ("db")
+		self.art_db.set_pixbuf (db, entry, pixbuf, self.on_get_pixbuf_completed)
+
+	def on_set_uri (self, widget, entry, uri):
+		db = self.shell.get_property ("db")
+		self.art_db.set_pixbuf_from_uri (db, entry, uri, self.on_get_pixbuf_completed)
