@@ -11,7 +11,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -45,7 +45,6 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-value.h>
 
-#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
 
 #include "eel-gconf-extensions.h"
@@ -57,7 +56,6 @@
 #include "rb-shell.h"
 #include "rb-shell-player.h"
 #include "rb-source.h"
-#include "md5.h"
 #include "rb-proxy-config.h"
 #include "rb-cut-and-paste-code.h"
 #include "rb-plugin.h"
@@ -181,13 +179,8 @@ static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 						char *url,
 						char *post_data,
 						SoupSessionCallback response_handler);
-#if defined(HAVE_LIBSOUP_2_4)
 static void	     rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
 static void	     rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
-#else
-static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
-static void	     rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data);
-#endif
 
 static void	     rb_audioscrobbler_import_settings (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler);
@@ -470,8 +463,9 @@ rb_audioscrobbler_add_timeout (RBAudioscrobbler *audioscrobbler)
 	if (!audioscrobbler->priv->timeout_id) {
 		rb_debug ("Adding Audioscrobbler timer (15 seconds)");
 		audioscrobbler->priv->timeout_id = 
-			g_timeout_add (15000, (GSourceFunc) rb_audioscrobbler_timeout_cb,
-				       audioscrobbler);
+			g_timeout_add_seconds (15,
+					       (GSourceFunc) rb_audioscrobbler_timeout_cb,
+					       audioscrobbler);
 	}
 }
 
@@ -549,22 +543,22 @@ rb_audioscrobbler_create_entry (RhythmDBEntry *rb_entry)
 	return as_entry;
 }
 
-static gboolean
+static void
 rb_audioscrobbler_add_to_queue (RBAudioscrobbler *audioscrobbler,
 				AudioscrobblerEntry *entry)
-{	
-	if (g_queue_get_length (audioscrobbler->priv->queue) < MAX_QUEUE_SIZE){
-		g_queue_push_tail (audioscrobbler->priv->queue, entry);
-		audioscrobbler->priv->queue_changed = TRUE;
-		audioscrobbler->priv->queue_count++;
-		return TRUE;
+{
+	if (g_queue_get_length (audioscrobbler->priv->queue) >= MAX_QUEUE_SIZE) {
+		AudioscrobblerEntry *oldest;
+
+		rb_debug ("queue limit reached.  dropping oldest entry.");
+		oldest = g_queue_pop_head (audioscrobbler->priv->queue);
+		rb_audioscrobbler_entry_free (oldest);
 	} else {
-		rb_debug ("Queue is too long.  Not adding song to queue");
-		g_free (audioscrobbler->priv->status_msg);
-		audioscrobbler->priv->status = QUEUE_TOO_LONG;
-		audioscrobbler->priv->status_msg = NULL;
-		return FALSE;
+		audioscrobbler->priv->queue_count++;
 	}
+
+	g_queue_push_tail (audioscrobbler->priv->queue, entry);
+	audioscrobbler->priv->queue_changed = TRUE;
 }
 
 static void
@@ -589,9 +583,8 @@ maybe_add_current_song_to_queue (RBAudioscrobbler *audioscrobbler)
 		if ((elapsed >= cur_entry->length / 2 || elapsed >= 240) && elapsed_delta < 20) {
 			rb_debug ("Adding currently playing song to queue");
 			time (&cur_entry->play_time);
-			if (rb_audioscrobbler_add_to_queue (audioscrobbler, cur_entry)){
-				audioscrobbler->priv->currently_playing = NULL;
-			}
+			rb_audioscrobbler_add_to_queue (audioscrobbler, cur_entry);
+			audioscrobbler->priv->currently_playing = NULL;
 			
 			rb_audioscrobbler_preferences_sync (audioscrobbler);
 		} else if (elapsed_delta > 20) {
@@ -650,26 +643,16 @@ rb_audioscrobbler_offline_play_notify_cb (RhythmDB *db,
 static gchar *
 mkmd5 (char *string)
 {
-	md5_state_t md5state;
-	guchar md5pword[16];
-	gchar md5_response[33];
-
-	int j = 0;
-
-	memset (md5_response, 0, sizeof (md5_response));
-
-	md5_init (&md5state);
-	md5_append (&md5state, (unsigned char*)string, strlen (string));
-	md5_finish (&md5state, md5pword);
-
-	for (j = 0; j < 16; j++) {
-		char a[3];
-		sprintf (a, "%02x", md5pword[j]);
-		md5_response[2*j] = a[0];
-		md5_response[2*j+1] = a[1];
-	}
-
-	return (g_strdup (md5_response));
+	GChecksum *checksum;
+	gchar *md5_result;
+	
+	checksum = g_checksum_new(G_CHECKSUM_MD5);	
+	g_checksum_update(checksum, (guchar *)string, -1);
+	
+	md5_result = g_strdup(g_checksum_get_string(checksum));
+	g_checksum_free(checksum);
+	
+	return (md5_result);
 }
 
 static void
@@ -679,27 +662,12 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 	rb_debug ("Parsing response, status=%d", msg->status_code);
 	
 	successful = FALSE;
-#if defined(HAVE_LIBSOUP_2_4)
 	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && msg->response_body->length != 0)
 		successful = TRUE;
-#else
-	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL)
-		successful = TRUE;
-#endif
 	if (successful) {
 		gchar **breaks;
 		int i;
-#if defined(HAVE_LIBSOUP_2_2)
-		gchar *body;
-
-		body = g_malloc0 ((msg->response).length + 1);
-		memcpy (body, (msg->response).body, (msg->response).length);
-
-		g_strstrip (body);
-		breaks = g_strsplit (body, "\n", 4);
-#else
 		breaks = g_strsplit (msg->response_body->data, "\n", 4);
-#endif
 
 		g_free (audioscrobbler->priv->status_msg);
 		audioscrobbler->priv->status = STATUS_OK;
@@ -771,9 +739,6 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 			audioscrobbler->priv->submit_next = time(NULL) + audioscrobbler->priv->submit_interval;
 
 		g_strfreev (breaks);
-#if defined(HAVE_LIBSOUP_2_2)
-		g_free (body);
-#endif
 	} else {
 		audioscrobbler->priv->status = REQUEST_FAILED;
 		audioscrobbler->priv->status_msg = g_strdup (msg->reason_phrase);
@@ -894,13 +859,8 @@ rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler)
 }
 
 
-#if defined(HAVE_LIBSOUP_2_4)
 static void
 rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
-#else
-static void
-rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
-#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER(user_data);
 
@@ -1057,13 +1017,8 @@ rb_g_queue_concat (GQueue *q1, GQueue *q2)
 	}
 }
 
-#if defined(HAVE_LIBSOUP_2_4)
 static void
 rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
-#else
-static void
-rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
-#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER (user_data);
 
@@ -1359,11 +1314,11 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 
 
 void
-rb_audioscrobbler_username_entry_changed_cb (GtkEntry *entry,
-					     RBAudioscrobbler *audioscrobbler)
+rb_audioscrobbler_username_entry_focus_out_event_cb (GtkWidget *widget,
+						     RBAudioscrobbler *audioscrobbler)
 {
 	eel_gconf_set_string (CONF_AUDIOSCROBBLER_USERNAME,
-			      gtk_entry_get_text (entry));
+			      gtk_entry_get_text (GTK_ENTRY (widget)));
 }
 
 void
@@ -1374,11 +1329,11 @@ rb_audioscrobbler_username_entry_activate_cb (GtkEntry *entry,
 }
 
 void
-rb_audioscrobbler_password_entry_changed_cb (GtkEntry *entry,
-					     RBAudioscrobbler *audioscrobbler)
+rb_audioscrobbler_password_entry_focus_out_event_cb (GtkWidget *widget,
+						     RBAudioscrobbler *audioscrobbler)
 {
 	eel_gconf_set_string (CONF_AUDIOSCROBBLER_PASSWORD,
-			      gtk_entry_get_text (entry));
+			      gtk_entry_get_text (GTK_ENTRY (widget)));
 }
 
 void
@@ -1394,101 +1349,97 @@ rb_audioscrobbler_password_entry_activate_cb (GtkEntry *entry,
 static gboolean
 rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 {
-	char *pathname, *uri;
-	GnomeVFSResult result;
+	char *pathname;
+	GFile *file;
+	GError *error = NULL;
 	char *data;
-	int size;
+	char *start;
+	char *end;
+	gsize size;
 
-	pathname = g_build_filename (rb_dot_dir (), "audioscrobbler.queue", NULL);
-	uri = g_filename_to_uri (pathname, NULL, NULL);
+	/* we don't really care about errors enough to report them here */
+	pathname = rb_find_user_data_file ("audioscrobbler.queue", NULL);
+	file = g_file_new_for_path (pathname);
+	rb_debug ("loading Audioscrobbler queue from \"%s\"", pathname);
 	g_free (pathname);
-	rb_debug ("Loading Audioscrobbler queue from \"%s\"", uri);
 
-	result = gnome_vfs_read_entire_file (uri, &size, &data);
-	g_free (uri);
-
-	/* do stuff */
-	if (result == GNOME_VFS_OK) {
-		char *start = data, *end;
-
-		/* scan along the file's data, turning each line into a string */
-		while (start < (data + size)) {
-			AudioscrobblerEntry *entry;
-
-			/* find the end of the line, to terminate the string */
-			end = g_utf8_strchr (start, -1, '\n');
-			if (end == NULL)
-				break;
-			*end = 0;
-
-			entry = rb_audioscrobbler_entry_load_from_string (start);
-			if (entry) {
-				g_queue_push_tail (audioscrobbler->priv->queue,
-						   entry);
-				audioscrobbler->priv->queue_count++;
-			}
-
-			start = end + 1;
-		}
+	if (g_file_load_contents (file, NULL, &data, &size, NULL, &error) == FALSE) {
+		rb_debug ("unable to load audioscrobbler queue: %s", error->message);
+		g_error_free (error);
+		return FALSE;
 	}
 
-	if (result != GNOME_VFS_OK) {
-		rb_debug ("Unable to load Audioscrobbler queue from disk: %s",
-			  gnome_vfs_result_to_string (result));
+	start = data;
+	while (start < (data + size)) {
+		AudioscrobblerEntry *entry;
+
+		/* find the end of the line, to terminate the string */
+		end = g_utf8_strchr (start, -1, '\n');
+		if (end == NULL)
+			break;
+		*end = 0;
+
+		entry = rb_audioscrobbler_entry_load_from_string (start);
+		if (entry) {
+			g_queue_push_tail (audioscrobbler->priv->queue,
+					   entry);
+			audioscrobbler->priv->queue_count++;
+		}
+
+		start = end + 1;
 	}
 
 	g_free (data);
-	return (result == GNOME_VFS_OK);
+	return TRUE;
 }
 
-
 static gboolean
 rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler)
 {
 	char *pathname;
-	GnomeVFSHandle *handle = NULL;
-	GnomeVFSResult result;
+	GFile *file;
+	GError *error = NULL;
+	GList *l;
+	GString *str;
 
 	if (!audioscrobbler->priv->queue_changed) {
 		return TRUE;
 	}
 
-	pathname = g_build_filename (rb_dot_dir (), "audioscrobbler.queue", NULL);
+	str = g_string_new ("");
+	for (l = audioscrobbler->priv->queue->head; l != NULL; l = g_list_next (l)) {
+		AudioscrobblerEntry *entry;
+
+		entry = (AudioscrobblerEntry *) l->data;
+		rb_audioscrobbler_entry_save_to_string (str, entry);
+	}
+
+	/* we don't really care about errors enough to report them here */
+	pathname = rb_find_user_data_file ("audioscrobbler.queue", NULL);
 	rb_debug ("Saving Audioscrobbler queue to \"%s\"", pathname);
 
-	result = gnome_vfs_create (&handle, pathname, GNOME_VFS_OPEN_WRITE, FALSE, 0600);
+	file = g_file_new_for_path (pathname);
 	g_free (pathname);
 
-	if (result == GNOME_VFS_OK) {
-		GString *s = g_string_new (NULL);
-		GList *l;
+	g_file_replace_contents (file,
+				 str->str, str->len,
+				 NULL,
+				 FALSE,
+				 G_FILE_CREATE_NONE,
+				 NULL,
+				 NULL,
+				 &error);
+	g_string_free (str, TRUE);
 
-		for (l = audioscrobbler->priv->queue->head;
-		     l != NULL;
-		     l = g_list_next (l)) {
-			AudioscrobblerEntry *entry;
-			char *str;
-			entry = (AudioscrobblerEntry *) l->data;
-			str = rb_audioscrobbler_entry_save_to_string (entry);
-			result = gnome_vfs_write (handle, str, strlen (str), 
-						  NULL);
-			g_free (str);
-			if (result != GNOME_VFS_OK)
-				break;
-		}
-		g_string_free (s, TRUE);
-	}
-
-	if (result != GNOME_VFS_OK) {
-		rb_debug ("Unable to save Audioscrobbler queue to disk: %s",
-			  gnome_vfs_result_to_string (result));
-	} else {
+	if (error == NULL) {
 		audioscrobbler->priv->queue_changed = FALSE;
+		return TRUE;
+	} else {
+		rb_debug ("error saving audioscrobbler queue: %s",
+			  error->message);
+		g_error_free (error);
+		return FALSE;
 	}
-
-	if (handle)
-		gnome_vfs_close (handle);
-	return (result == GNOME_VFS_OK);
 }
 
 static void
