@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,46 +35,44 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#ifdef HAVE_HAL
-#include <libhal.h>
-#include <dbus/dbus.h>
-#endif
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gpod/itdb.h>
 
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
 #include "rb-ipod-db.h"
+#include "rb-ipod-helpers.h"
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
+#include "rb-glade-helpers.h"
 #include "rb-plugin.h"
 #include "rb-removable-media-manager.h"
-#include "rb-static-playlist-source.h"
+#include "rb-ipod-static-playlist-source.h"
 #include "rb-util.h"
 #include "rhythmdb.h"
 #include "rb-cut-and-paste-code.h"
 
+#define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
+#define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
+
 static GObject *rb_ipod_source_constructor (GType type, 
 					    guint n_construct_properties,
 					    GObjectConstructParam *construct_properties);
 static void rb_ipod_source_dispose (GObject *object);
 
+static char *impl_get_browser_key (RBSource *source);
+static char *impl_get_paned_key (RBBrowserSource *source);
+
 static gboolean impl_show_popup (RBSource *source);
 static void impl_move_to_trash (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
-#ifdef HAVE_HAL
-static gboolean hal_udi_is_ipod (const char *udi);
-#endif
 
-#ifdef ENABLE_IPOD_WRITING
 static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *dest,
+				  guint64 filesize,
 				  const char *mimetype);
 static char* impl_build_dest_uri (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
@@ -83,7 +89,7 @@ static gboolean rb_ipod_song_artwork_add_cb (RhythmDB *db,
                                              const gchar *property_name,
                                              const GValue *metadata,
                                              RBiPodSource *isource);
-#endif
+
 static RhythmDB *get_db_for_source (RBiPodSource *source);
 
 struct _PlayedEntry {
@@ -99,7 +105,7 @@ typedef struct
 	GHashTable *entry_map;
 
 	gboolean needs_shuffle_db;
-	RBStaticPlaylistSource *podcast_pl;
+	RBIpodStaticPlaylistSource *podcast_pl;
 
 	guint load_idle_id;
 
@@ -109,12 +115,10 @@ typedef struct
 	GQueue *offline_plays;
 } RBiPodSourcePrivate;
 
-#ifdef ENABLE_IPOD_WRITING
 typedef struct {
 	RBiPodSourcePrivate *priv;
 	GdkPixbuf *pixbuf;
 } RBiPodSongArtworkAddData;
-#endif
 
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
 		      rb_ipod_source,
@@ -128,10 +132,13 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->constructor = rb_ipod_source_constructor;
 	object_class->dispose = rb_ipod_source_dispose;
 
+	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
@@ -139,15 +146,13 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
-#ifdef ENABLE_IPOD_WRITING
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
+	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
 	rms_class->impl_track_added = impl_track_added;
 	rms_class->impl_build_dest_uri = impl_build_dest_uri;
 	rms_class->impl_get_mime_types = impl_get_mime_types;
-#else
-	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
-	rms_class->impl_track_added = NULL;
-#endif
+
+	browser_source_class->impl_get_paned_key = impl_get_paned_key;
 
 	g_type_class_add_private (klass, sizeof (RBiPodSourcePrivate));
 }
@@ -158,6 +163,11 @@ rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't change ipod name with no ipod db");
+		return;
+	}
+
 	rb_ipod_db_set_ipod_name (priv->ipod_db, name);
 }
 
@@ -175,8 +185,6 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 static void
 rb_ipod_source_init (RBiPodSource *source)
 {	
-	g_signal_connect (G_OBJECT (source), "notify::name",
-			  (GCallback)rb_ipod_source_name_changed_cb, NULL);
 }
 
 static GObject *
@@ -188,20 +196,20 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 
 	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
+
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
+        rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
 
 	rb_ipod_load_songs (source);
 
-#ifdef ENABLE_IPOD_WRITING                                      
         RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (source));
         g_signal_connect_object (db,
                                  "entry-extra-metadata-notify::rb:coverArt",
                                  G_CALLBACK (rb_ipod_song_artwork_add_cb),
                                  RB_IPOD_SOURCE(source), 0);
         g_object_unref (G_OBJECT (db));
-#endif
 
 	return G_OBJECT (source);
 }
@@ -249,19 +257,22 @@ rb_ipod_source_dispose (GObject *object)
 }
 
 RBRemovableMediaSource *
-rb_ipod_source_new (RBShell *shell,
-		    GnomeVFSVolume *volume)
+rb_ipod_source_new (RBPlugin *plugin,
+		    RBShell *shell,
+		    GMount *mount)
 {
 	RBiPodSource *source;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db;
+	GVolume *volume;
 	char *name;
 	char *path;
 
-	g_assert (rb_ipod_is_volume_ipod (volume));
-
+	volume = g_mount_get_volume (mount);
+	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	g_object_unref (volume);
+	
 	g_object_get (shell, "db", &db, NULL);
-	path = gnome_vfs_volume_get_device_path (volume);
 	name = g_strdup_printf ("ipod: %s", path);
 	entry_type =  rhythmdb_entry_register_type (db, name);
 	entry_type->save_to_disk = FALSE;
@@ -271,8 +282,9 @@ rb_ipod_source_new (RBShell *shell,
 	g_free (path);
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
+				               "plugin", plugin,
 					       "entry-type", entry_type,
-					       "volume", volume,
+					       "mount", mount,
 					       "shell", shell,
 					       "source-group", RB_SOURCE_GROUP_DEVICES,
 					       NULL));
@@ -319,16 +331,16 @@ playlist_track_removed (RhythmDBQueryModel *m,
 			RhythmDBEntry *entry,
 			gpointer data)
 {
-	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
-	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
-	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
 	Itdb_Track *track;
 
 	g_return_if_fail (ipod != NULL);
 	g_return_if_fail (ipod_pl != NULL);
 
-        track = g_hash_table_lookup (priv->entry_map, entry);
+	track = g_hash_table_lookup (priv->entry_map, entry);
 	g_return_if_fail (track != NULL);
 	rb_ipod_db_remove_from_playlist (priv->ipod_db, ipod_pl, track);
 }
@@ -337,9 +349,9 @@ static void
 playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
 		      GtkTreeIter *iter, gpointer data)
 {
-	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
-	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
-	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
 	Itdb_Track *track;
 	RhythmDBEntry *entry;
@@ -358,7 +370,7 @@ static void
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
 	RBShell *shell;
-	RBSource *playlist_source;
+	RBIpodStaticPlaylistSource *playlist_source;
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
@@ -369,10 +381,11 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 			  "entry-type", &entry_type,
 			  NULL);
 
-	playlist_source = rb_static_playlist_source_new (shell,
-							 playlist->name,
-							 FALSE,
-							 entry_type);
+	playlist_source = rb_ipod_static_playlist_source_new (shell,
+                                                              source,
+                                                              priv->ipod_db,
+                                                              playlist,
+                                                              entry_type);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	for (it = playlist->members; it != NULL; it = it->next) {
@@ -388,7 +401,11 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 		g_free (filename);
 	}
 
-	g_object_ref (G_OBJECT(playlist_source));
+        /* RBSource derives from GtkWidget so its initial reference is
+         * floating. Since we need a ref for ourselves and we don't want it to
+         * be stolen by a GtkContainer, we sink the floating reference.
+         */
+	g_object_ref_sink (G_OBJECT(playlist_source));
 	playlist->userdata = playlist_source;
 	playlist->userdata_destroy = g_object_unref;
 	playlist->userdata_duplicate = g_object_ref;
@@ -401,13 +418,9 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 			          G_CALLBACK (playlist_track_removed),
 			          playlist_source);
 
-	g_object_set_data (G_OBJECT (playlist_source),
-			           "ipod-source", source);
-	g_object_set_data (G_OBJECT (playlist_source),
-			           "itdb-playlist", playlist);
 	if (itdb_playlist_is_podcasts(playlist))
-			priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
-	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
+		priv->podcast_pl = playlist_source;
+	rb_shell_append_source (shell, RB_SOURCE (playlist_source), RB_SOURCE (source));
 	g_object_unref (shell);
 }
 
@@ -425,8 +438,7 @@ load_ipod_playlists (RBiPodSource *source)
 		playlist = (Itdb_Playlist *)it->data;
 		if (itdb_playlist_is_mpl (playlist)) {
 			continue;
-		}
-		if (playlist->is_spl) {
+		} else if (playlist->is_spl) {
 			continue;
 		}
 
@@ -435,8 +447,6 @@ load_ipod_playlists (RBiPodSource *source)
 
 }
 
-#ifdef ENABLE_IPOD_WRITING
-
 /* FIXME:  these should go away once we compile against new-enough libgpod */
 #define MEDIATYPE_AUDIO         0x0001
 #define MEDIATYPE_PODCAST       0x0004
@@ -451,6 +461,8 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	track->album = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ALBUM);
 	track->artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
+	track->sort_artist = rhythmdb_entry_dup_string (entry,
+	                                                RHYTHMDB_PROP_ARTIST_SORTNAME);
 	track->genre = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_GENRE);
 	track->filetype = g_strdup (mimetype);
 	track->size = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
@@ -459,10 +471,9 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->cd_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 	track->track_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	track->bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE);
-	track->year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DATE);
-	track->time_added = itdb_time_get_mac_time ();
+	track->year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
+	track->time_added = time (NULL);
 	track->time_played = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_PLAYED);
-	track->time_played = itdb_time_host_to_mac (track->time_played);
 	track->rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
 	track->rating *= ITDB_RATING_STEP;
 	track->app_rating = track->rating;
@@ -470,13 +481,13 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 
 	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
 		track->mediatype = MEDIATYPE_PODCAST;
+		track->time_released = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
 	} else {
 		track->mediatype = MEDIATYPE_AUDIO;
 	}
 
 	return track;
 }
-#endif
 
 static void add_offline_played_entry (RBiPodSource *source,
 				      RhythmDBEntry *entry,
@@ -630,7 +641,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->time_added != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
-		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_added));
+		g_value_set_ulong (&value, song->time_added);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_FIRST_SEEN,
 					       &value);
@@ -641,7 +652,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->time_played != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
-		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_played));
+		g_value_set_ulong (&value, song->time_played);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_LAST_PLAYED,
 					       &value);
@@ -656,6 +667,12 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	entry_set_string_prop (RHYTHMDB (db), entry,
 			       RHYTHMDB_PROP_ARTIST, song->artist);
 
+        if (song->sort_artist != NULL) {
+                entry_set_string_prop (RHYTHMDB (db), entry,
+                                       RHYTHMDB_PROP_ARTIST_SORTNAME,
+                                       song->sort_artist);
+        }
+
 	entry_set_string_prop (RHYTHMDB (db), entry,
 			       RHYTHMDB_PROP_ALBUM, song->album);
 
@@ -851,7 +868,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			if (old_lastplay != new_lastplay) {
 				track = g_hash_table_lookup (priv->entry_map, 
 							     entry);
-				track->time_played = itdb_time_host_to_mac (new_lastplay);
+				track->time_played = new_lastplay;
 				rb_debug ("last play time changed, saving db");
 				rb_ipod_db_save_async (priv->ipod_db);
 			} else {
@@ -902,10 +919,10 @@ static void
 rb_ipod_load_songs (RBiPodSource *source)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-	GnomeVFSVolume *volume;
+	GMount *mount;
 
-	g_object_get (source, "volume", &volume, NULL);
- 	priv->ipod_db = rb_ipod_db_new (volume);
+	g_object_get (source, "mount", &mount, NULL);
+ 	priv->ipod_db = rb_ipod_db_new (mount);
 	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
@@ -916,150 +933,14 @@ rb_ipod_load_songs (RBiPodSource *source)
 				      "name", name,
 				      NULL);
 		}
+                g_signal_connect (G_OBJECT (source), "notify::name",
+		  	          (GCallback)rb_ipod_source_name_changed_cb,
+                                  NULL);
 		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
-	g_object_unref (volume);
+	g_object_unref (mount);
 }
 
-static gchar *
-rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
-{
-	gchar *mount_point_uri;
-	gchar *mount_point;
-	gchar *result;
-
-	mount_point_uri = gnome_vfs_volume_get_activation_uri (volume);
-	if (mount_point_uri == NULL) {
-		return NULL;
-	}
-	mount_point = g_filename_from_uri (mount_point_uri, NULL, NULL);
-	g_free (mount_point_uri);
-	if (mount_point == NULL) {
-		return NULL;
-	}
-
-	result = itdb_get_itunes_dir(mount_point);
-
-	g_free (mount_point);
-	return result;
-}
-
-static gboolean
-rb_ipod_volume_has_ipod_db (GnomeVFSVolume *volume)
-{
-	char *itunesdb_path;
-	gboolean result;
-
-	itunesdb_path = rb_ipod_get_itunesdb_path (volume);
-
-	if (itunesdb_path != NULL) {
-		result = g_file_test (itunesdb_path, G_FILE_TEST_EXISTS);
-	} else {
-		result = FALSE;
-	}
-	g_free (itunesdb_path);
-
-	return result;
-}
-
-gboolean
-rb_ipod_is_volume_ipod (GnomeVFSVolume *volume)
-{
-#ifdef HAVE_HAL
-	gchar *udi;
-#endif
-	if (gnome_vfs_volume_get_volume_type (volume) != GNOME_VFS_VOLUME_TYPE_MOUNTPOINT) {
-		return FALSE;
-	}
-
-#ifdef HAVE_HAL
-	udi = gnome_vfs_volume_get_hal_udi (volume);
-	if (udi != NULL) {
-		gboolean result;
-
-		result = hal_udi_is_ipod (udi);
-		g_free (udi);
-		if (result == FALSE) {
-			return FALSE;
-		}
-	}
-#endif
-
-	return rb_ipod_volume_has_ipod_db (volume);
-}
-
-#ifdef HAVE_HAL
-
-static gboolean
-hal_udi_is_ipod (const char *udi)
-{
-	LibHalContext *ctx;
-	DBusConnection *conn;
-	char *parent_udi;
-	char *parent_name;
-	gboolean result;
-	DBusError error;
-	gboolean inited = FALSE;
-
-	result = FALSE;
-	dbus_error_init (&error);
-
-	conn = NULL;
-	parent_udi = NULL;
-	parent_name = NULL;
-
-	ctx = libhal_ctx_new ();
-	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can
-		 * fall back to a check for iTunesDB presence instead ?
-		 */
-		rb_debug ("cannot connect to HAL");
-		goto end;
-	}
-	conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (conn == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	libhal_ctx_set_dbus_connection (ctx, conn);
-	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
-		goto end;
-
-	inited = TRUE;
-	parent_udi = libhal_device_get_property_string (ctx, udi,
-			"info.parent", &error);
-	if (parent_udi == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	parent_name = libhal_device_get_property_string (ctx, parent_udi,
-			"storage.model", &error);
-	if (parent_name == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	if (strcmp (parent_name, "iPod") == 0)
-		result = TRUE;
-
-end:
-	g_free (parent_udi);
-	g_free (parent_name);
-
-	if (dbus_error_is_set (&error)) {
-		rb_debug ("Error: %s\n", error.message);
-		dbus_error_free (&error);
-		dbus_error_init (&error);
-	}
-
-	if (ctx) {
-		if (inited)
-			libhal_ctx_shutdown (ctx, &error);
-		libhal_ctx_free(ctx);
-	}
-
-	dbus_error_free (&error);
-
-	return result;
-}
-#endif
-
 static GList*
 impl_get_ui_actions (RBSource *source)
 {
@@ -1070,6 +951,18 @@ impl_get_ui_actions (RBSource *source)
 	return actions;
 }
 
+static char *
+impl_get_browser_key (RBSource *source)
+{
+	return g_strdup (CONF_STATE_SHOW_BROWSER);
+}
+
+static char *
+impl_get_paned_key (RBBrowserSource *source)
+{
+	return g_strdup (CONF_STATE_PANED_POSITION);
+}
+
 static gboolean
 impl_show_popup (RBSource *source)
 {
@@ -1120,7 +1013,6 @@ impl_move_to_trash (RBSource *asource)
 	g_list_free (sel);
 }
 
-#ifdef ENABLE_IPOD_WRITING
 static char *
 impl_build_dest_uri (RBRemovableMediaSource *source,
 		     RhythmDBEntry *entry,
@@ -1132,6 +1024,10 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	char *dest;
 	const char *mount_path;
 
+	if (priv->ipod_db == NULL) {
+		return NULL;
+	}
+
 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
 	dest = ipod_get_filename_for_uri (mount_path,  uri, 
@@ -1225,6 +1121,10 @@ rb_ipod_song_artwork_add_cb (RhythmDB *db,
 		return FALSE;
 	}
 
+	if (song->album == NULL || song->artist == NULL) {
+		return FALSE;
+	}
+
         g_object_get (G_OBJECT (isource), "entry-type", &entry_type, NULL);
  
 	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
@@ -1301,7 +1201,7 @@ add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
 
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
   	filename = ipod_path_to_uri (mount_path, song->ipod_path);
- 	rb_static_playlist_source_add_location (priv->podcast_pl, filename, -1);
+ 	rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (priv->podcast_pl), filename, -1);
 	g_free (filename);
 }
 
@@ -1309,6 +1209,7 @@ static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
 		  const char *dest,
+		  guint64 filesize,
 		  const char *mimetype)
 {
 	RBiPodSource *isource = RB_IPOD_SOURCE (source);
@@ -1322,6 +1223,7 @@ impl_track_added (RBRemovableMediaSource *source,
 		RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 		char *filename;
 		const char *mount_path;
+		Itdb_Device *device;
 
 		filename = g_filename_from_uri (dest, NULL, NULL);
 		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
@@ -1332,18 +1234,10 @@ impl_track_added (RBRemovableMediaSource *source,
 		if (song->mediatype == MEDIATYPE_PODCAST) {
 			add_to_podcasts (isource, song);
 		}
-#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
-		/* reuse that #define since both functions were added to 
-		 * libgpod CVS HEAD around the same time
-		 */
-		Itdb_Device *device;
 		device = rb_ipod_db_get_device (priv->ipod_db);		
 		if (device && itdb_device_supports_artwork (device)) {
 			request_artwork (isource, entry, db, song);
 		}
-#else 
-		request_artwork (isource, entry, db, song);
-#endif
 		add_ipod_song_to_db (isource, db, song);
 		rb_ipod_db_add_track (priv->ipod_db, song);
 	}
@@ -1528,7 +1422,7 @@ ipod_get_filename_for_uri (const gchar *mount_point,
 	if (escaped == NULL) {
 		return NULL;
 	}
-	filename = gnome_vfs_unescape_string (escaped, G_DIR_SEPARATOR_S);
+	filename = g_uri_unescape_string (escaped, NULL);
 	g_free (escaped);
 	if (filename == NULL) {
 		return NULL;
@@ -1581,7 +1475,6 @@ ipod_path_from_unix_path (const gchar *mount_point, const gchar *unix_path)
 
 	return ipod_path;
 }
-#endif
 
 static void
 impl_delete_thyself (RBSource *source)
@@ -1589,6 +1482,11 @@ impl_delete_thyself (RBSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *p;
 
+        if (priv->ipod_db == NULL) {
+            RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
+            return;
+        }
+
 	for (p = rb_ipod_db_get_playlists (priv->ipod_db);
 	     p != NULL;
 	     p = p->next) {
@@ -1611,16 +1509,12 @@ impl_delete_thyself (RBSource *source)
 		}
 	}
 
-	if (priv->ipod_db != NULL) {
-		g_object_unref (G_OBJECT (priv->ipod_db));
-		priv->ipod_db = NULL;
-	}
+	g_object_unref (G_OBJECT (priv->ipod_db));
+	priv->ipod_db = NULL;
 
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
 
-#ifdef ENABLE_IPOD_WRITING
-
 static GList *
 impl_get_mime_types (RBRemovableMediaSource *source)
 {
@@ -1633,4 +1527,153 @@ impl_get_mime_types (RBRemovableMediaSource *source)
 	return ret;
 }
 
-#endif
+
+void
+rb_ipod_source_new_playlist (RBiPodSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	Itdb_Playlist *ipod_playlist;
+
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't create new ipod playlist with no ipod db");
+		return;
+	}
+
+	ipod_playlist = itdb_playlist_new (_("New playlist"), FALSE);
+	rb_ipod_db_add_playlist (priv->ipod_db, ipod_playlist);
+	add_rb_playlist (source, ipod_playlist);
+}
+
+void
+rb_ipod_source_remove_playlist (RBiPodSource *ipod_source,
+				RBSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod_source);
+	RBIpodStaticPlaylistSource *playlist_source = RB_IPOD_STATIC_PLAYLIST_SOURCE (source);
+
+	rb_source_delete_thyself (source);
+	rb_ipod_db_remove_playlist (priv->ipod_db, rb_ipod_static_playlist_source_get_itdb_playlist (playlist_source));
+}
+
+static void
+rb_ipod_info_response_cb (GtkDialog *dialog,
+ 			  int response_id,
+ 			  RBiPodSource *source)
+{
+	if (response_id == GTK_RESPONSE_CLOSE) {
+ 		
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+	}
+}
+
+static gboolean
+ipod_name_changed_cb (GtkWidget     *widget,
+ 		      GdkEventFocus *event,
+ 		      gpointer       user_data) 
+{
+	g_object_set (RB_SOURCE (user_data), "name", 
+		      gtk_entry_get_text (GTK_ENTRY (widget)),
+		      NULL);
+	return FALSE;
+}
+
+void
+rb_ipod_source_show_properties (RBiPodSource *source)
+{
+	GladeXML *xml;
+	GtkWidget *dialog;
+	GtkWidget *label;
+	char *text;
+	const gchar *mp;
+	char *used;
+	char *capacity;
+	char *glade_file;
+ 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	Itdb_Device *ipod_dev;
+	RBPlugin *plugin;
+
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't show ipod properties with no ipod db");
+		return;
+	}
+
+	ipod_dev = rb_ipod_db_get_device (priv->ipod_db);
+
+	g_object_get (source, "plugin", &plugin, NULL);
+	glade_file = rb_plugin_find_file (plugin, "ipod-info.glade");
+	g_object_unref (plugin);
+
+	if (glade_file == NULL) {
+		g_warning ("Couldn't find ipod-info.glade");
+		return;
+	}
+
+	xml = rb_glade_xml_new (glade_file, "ipod-information", NULL);
+	g_free (glade_file);
+
+ 	if (xml == NULL) {
+ 		rb_debug ("Couldn't load ipod-info.glade");
+ 		return;
+ 	}
+	
+ 	dialog = glade_xml_get_widget (xml, "ipod-information");
+ 	g_signal_connect_object (G_OBJECT (dialog),
+ 				 "response",
+ 				 G_CALLBACK (rb_ipod_info_response_cb),
+ 				 source, 0);
+ 
+ 	label = glade_xml_get_widget (xml, "label-number-track-number");
+ 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_tracks(priv->ipod_db) ));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+ 	g_free (text);
+ 
+ 	label = glade_xml_get_widget (xml, "entry-ipod-name");
+ 	gtk_entry_set_text (GTK_ENTRY (label), rb_ipod_db_get_ipod_name (priv->ipod_db));
+ 	g_signal_connect (G_OBJECT (label), "focus-out-event", 
+ 			  (GCallback)ipod_name_changed_cb, source);
+ 
+ 	label = glade_xml_get_widget (xml, "label-number-playlist-number");
+ 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_playlists (priv->ipod_db)));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+ 	g_free (text);
+ 
+ 	label = glade_xml_get_widget (xml, "label-mount-point-value");
+	mp = rb_ipod_db_get_mount_path (priv->ipod_db);
+ 	gtk_label_set_text (GTK_LABEL (label), mp);
+
+	label = glade_xml_get_widget (xml, "progressbar-ipod-usage");
+	used = g_format_size_for_display (rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp));
+	capacity = g_format_size_for_display (rb_ipod_helpers_get_capacity(mp));
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (label), 
+				       (double)(rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp))/(double)rb_ipod_helpers_get_capacity (mp));
+	/* Translators: this is used to display the amount of storage space
+	 * used and the total storage space on an iPod.
+	 */
+	text = g_strdup_printf (_("%s of %s"), used, capacity);
+	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (label), text);
+	g_free (text);
+	g_free (capacity);
+	g_free (used);
+
+	label = glade_xml_get_widget (xml, "label-device-node-value");
+	text = rb_ipod_helpers_get_device (RB_SOURCE(source));
+	gtk_label_set_text (GTK_LABEL (label), text);
+	g_free (text);
+
+ 	label = glade_xml_get_widget (xml, "label-ipod-model-value");
+ 	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "ModelNumStr"));
+
+ 	label = glade_xml_get_widget (xml, "label-database-version-value");
+	text = g_strdup_printf ("%u", rb_ipod_db_get_database_version (priv->ipod_db));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+	g_free (text);
+
+ 	label = glade_xml_get_widget (xml, "label-serial-number-value");
+	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "pszSerialNumber"));
+
+ 	label = glade_xml_get_widget (xml, "label-firmware-version-value");
+	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "VisibleBuildID"));
+
+ 	gtk_widget_show (GTK_WIDGET (dialog));
+}
+
