@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,13 +36,12 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <glib.h>
 
 #include "rb-feed-podcast-properties-dialog.h"
 #include "rb-file-helpers.h"
 #include "rb-glade-helpers.h"
 #include "rb-dialog.h"
-#include "rb-rating.h"
 #include "rb-cut-and-paste-code.h"
 #include "rhythmdb.h"
 
@@ -235,8 +242,10 @@ rb_feed_podcast_properties_dialog_update_location (RBFeedPodcastPropertiesDialog
 	const char *location;
 	char *unescaped;
 
-	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
-	unescaped = gnome_vfs_unescape_string_for_display (location);
+	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_MOUNTPOINT);
+	if (location == NULL)
+		location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
+	unescaped = g_uri_unescape_string (location, NULL);
 	gtk_label_set_text (GTK_LABEL (dialog->priv->location), unescaped);
 	g_free (unescaped);
 }
@@ -267,7 +276,7 @@ rb_feed_podcast_properties_dialog_update_last_update (RBFeedPodcastPropertiesDia
 
 	time_val = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_LAST_SEEN);
 	time_str = rb_feed_podcast_properties_dialog_parse_time (time_val);
-	gtk_label_set (GTK_LABEL (dialog->priv->last_update), time_str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->last_update), time_str);
 	g_free (time_str);
 }
 
@@ -279,7 +288,7 @@ rb_feed_podcast_properties_dialog_update_last_episode (RBFeedPodcastPropertiesDi
 
 	time_val = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_POST_TIME);
 	time_str = rb_feed_podcast_properties_dialog_parse_time (time_val);
-	gtk_label_set (GTK_LABEL (dialog->priv->last_episode), time_str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->last_episode), time_str);
 	g_free (time_str);
 }
 
@@ -299,7 +308,7 @@ rb_feed_podcast_properties_dialog_update_summary (RBFeedPodcastPropertiesDialog
 						     RHYTHMDB_PROP_SUBTITLE);
 	}
 
-	gtk_label_set (GTK_LABEL (dialog->priv->summary), summary);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->summary), summary);
 }
 
 static char *
