@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -25,14 +33,25 @@
 #include <string.h>
 
 #include <totem-pl-parser.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
+#include <gio/gio.h>
+#include <glib.h>
+#include <glib/gprintf.h>
 
 #include "rb-debug.h"
 #include "rb-podcast-parse.h"
 #include "rb-file-helpers.h"
 
+GQuark
+rb_podcast_parse_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("rb_podcast_parse_error");
+
+	return quark;
+}
+
 static void
 playlist_metadata_foreach (const char *key,
 			   const char *value,
@@ -118,10 +137,12 @@ entry_parsed (TotemPlParser *parser,
 
 gboolean
 rb_podcast_parse_load_feed (RBPodcastChannel *data,
-			    const char *file_name)
+			    const char *file_name,
+			    gboolean existing_feed,
+			    GError **error)
 {
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *info;
+	GFile *file;
+	GFileInfo *fileinfo;
 	TotemPlParser *plparser;
 
 	data->url = g_strdup (file_name);
@@ -129,82 +150,82 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	/* if the URL has a .rss, .xml or .atom extension (before the query string),
 	 * don't bother checking the MIME type.
 	 */
-	if (rb_uri_could_be_podcast (file_name, &data->is_opml)) {
+	if (rb_uri_could_be_podcast (file_name, &data->is_opml) || existing_feed) {
 		rb_debug ("not checking mime type for %s (should be %s file)", file_name,
 			  data->is_opml ? "OPML" : "Podcast");
 	} else {
-		gboolean invalid_mime_type;
+		GError *ferror = NULL;
+		char *content_type;
 
 		rb_debug ("checking mime type for %s", file_name);
-		info = gnome_vfs_file_info_new ();
 
-		result = gnome_vfs_get_file_info (file_name, info, GNOME_VFS_FILE_INFO_DEFAULT);
-
-		if ((result != GNOME_VFS_OK)) {
-			if (info->mime_type != NULL) {
-				rb_debug ("Invalid mime-type in podcast feed %s", info->mime_type);
-			} else {
-				rb_debug ("Couldn't get mime type for %s: %s", file_name,
-					  gnome_vfs_result_to_string (result));
-			}
-			gnome_vfs_file_info_unref (info);
-			return TRUE;
-		}
-
-		if (info != NULL
-		    && info->mime_type != NULL
-		    && strstr (info->mime_type, "html") == NULL
-		    && strstr (info->mime_type, "xml") == NULL
-		    && strstr (info->mime_type, "rss") == NULL
-		    && strstr (info->mime_type, "opml") == NULL) {
-			invalid_mime_type = TRUE;
-		} else if (info != NULL
-			   && info->mime_type != NULL
-			   && strstr (info->mime_type, "opml") != NULL) {
-			data->is_opml = TRUE;
-			invalid_mime_type = FALSE;
-		} else {
-			invalid_mime_type = FALSE;
-		}
-
-		if (invalid_mime_type) {
-			GtkWidget *dialog;
-
-			GDK_THREADS_ENTER ();
-			dialog = gtk_message_dialog_new (NULL, 0,
-							 GTK_MESSAGE_QUESTION,
-							 GTK_BUTTONS_YES_NO,
-							 _("The URL '%s' does not appear to be a podcast feed. "
-							 "It may be the wrong URL, or the feed may be broken. "
-							 "Would you like Rhythmbox to attempt to use it anyway?"),
-							 file_name);
-
-			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-				invalid_mime_type = FALSE;
-
-			gtk_widget_destroy (dialog);
-			GDK_THREADS_LEAVE ();
-		}
-
-		gnome_vfs_file_info_unref (info);
-
-		if (invalid_mime_type)
+		file = g_file_new_for_uri (file_name);
+		fileinfo = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, 0, NULL, &ferror);
+		if (ferror != NULL) {
+			g_set_error (error,
+				     RB_PODCAST_PARSE_ERROR,
+				     RB_PODCAST_PARSE_ERROR_FILE_INFO,
+				     _("Unable to check file type: %s"),
+				     ferror->message);
+			g_object_unref (file);
+			g_clear_error (&ferror);
 			return FALSE;
+		}
+
+		content_type = g_file_info_get_attribute_as_string (fileinfo, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+		g_object_unref (file);
+		g_object_unref (fileinfo);
+
+		if (content_type != NULL
+		    && strstr (content_type, "html") == NULL
+		    && strstr (content_type, "xml") == NULL
+		    && strstr (content_type, "rss") == NULL
+		    && strstr (content_type, "opml") == NULL) {
+			g_set_error (error,
+				     RB_PODCAST_PARSE_ERROR,
+				     RB_PODCAST_PARSE_ERROR_MIME_TYPE,
+				     _("Unexpected file type: %s"),
+				     content_type);
+			g_free (content_type);
+			return FALSE;
+		} else if (content_type != NULL
+			   && strstr (content_type, "opml") != NULL) {
+			data->is_opml = TRUE;
+		}
+
+		g_free (content_type);
 	}
 
 	plparser = totem_pl_parser_new ();
-	g_object_set (plparser, "recurse", FALSE, NULL);
+	g_object_set (plparser, "recurse", FALSE, "force", TRUE, NULL);
 	g_signal_connect (G_OBJECT (plparser), "entry-parsed", G_CALLBACK (entry_parsed), data);
 	g_signal_connect (G_OBJECT (plparser), "playlist-started", G_CALLBACK (playlist_started), data);
 	g_signal_connect (G_OBJECT (plparser), "playlist-ended", G_CALLBACK (playlist_ended), data);
 
 	if (totem_pl_parser_parse (plparser, file_name, FALSE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 		rb_debug ("Parsing %s as a Podcast failed", file_name);
+		g_set_error (error,
+			     RB_PODCAST_PARSE_ERROR,
+			     RB_PODCAST_PARSE_ERROR_XML_PARSE,
+			     _("Unable to parse the feed contents"));
 		g_object_unref (plparser);
 		return FALSE;
 	}
-	rb_debug ("Parsing %s as a Podcast succeeded", file_name);
+	g_object_unref (plparser);
 
+	/* treat empty feeds, or feeds that don't contain any downloadable items, as
+	 * an error.
+	 */
+	if (data->posts == NULL) {
+		rb_debug ("Parsing %s as a podcast succeeded, but the feed contains no downloadable items", file_name);
+		g_set_error (error,
+			     RB_PODCAST_PARSE_ERROR,
+			     RB_PODCAST_PARSE_ERROR_NO_ITEMS,
+			     _("The feed does not contain any downloadable items"));
+		return FALSE;
+	}
+
+	rb_debug ("Parsing %s as a Podcast succeeded", file_name);
 	return TRUE;
 }
 
