@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,7 +36,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <glib.h>
 
 #include "rb-podcast-properties-dialog.h"
 #include "rb-file-helpers.h"
@@ -137,6 +137,7 @@ static void
 rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 {
 	GladeXML *xml;
+	AtkObject *lobj, *robj;
 
 	dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
 						    RB_TYPE_PODCAST_PROPERTIES_DIALOG,
@@ -197,6 +198,14 @@ rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 				 G_OBJECT (dialog), 0);
 	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "ratingVBox")),
 			   dialog->priv->rating);
+
+	/* add relationship between the rating label and the rating widget */
+	lobj = gtk_widget_get_accessible (glade_xml_get_widget (xml, "ratingDescLabel"));
+	robj = gtk_widget_get_accessible (dialog->priv->rating);
+	
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
+
 	g_object_unref (G_OBJECT (xml));
 }
 
@@ -402,8 +411,10 @@ rb_podcast_properties_dialog_update_location (RBPodcastPropertiesDialog *dialog)
 	const char *location;
 	char *display;
 
-	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
-	display = gnome_vfs_format_uri_for_display (location);
+	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_MOUNTPOINT);
+	if (location == NULL)
+		location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
+	display = g_uri_unescape_string (location, NULL);
 	gtk_label_set_text (GTK_LABEL (dialog->priv->location), display);
 	g_free (display);
 }
@@ -416,7 +427,8 @@ rb_podcast_properties_dialog_update_download_location (RBPodcastPropertiesDialog
 	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_MOUNTPOINT);
 	if (location != NULL && location[0] != '\0') {
 		char *display;
-		display = gnome_vfs_format_uri_for_display (location);
+		location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
+		display = g_uri_unescape_string (location, NULL);
 		gtk_label_set_text (GTK_LABEL (dialog->priv->download_location), display);
 		g_free (display);
 	} else {
@@ -484,7 +496,7 @@ rb_podcast_properties_dialog_update_last_played (RBPodcastPropertiesDialog *dial
 	const char *str;
 
 	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LAST_PLAYED_STR);
-	gtk_label_set (GTK_LABEL (dialog->priv->lastplayed), str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->lastplayed), str);
 }
 
 static void
@@ -505,8 +517,7 @@ rb_podcast_properties_dialog_update_date (RBPodcastPropertiesDialog *dialog)
 	post_time = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_POST_TIME);
 	time = rb_podcast_properties_dialog_parse_time (post_time);
 
-	gtk_label_set (GTK_LABEL (dialog->priv->date),
-		       time);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->date), time);
 	g_free (time);
 }
 
@@ -516,7 +527,7 @@ rb_podcast_properties_dialog_update_description (RBPodcastPropertiesDialog *dial
 	const char *str;
 
 	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_DESCRIPTION);
-	gtk_label_set (GTK_LABEL (dialog->priv->description), str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->description), str);
 }
 
 static char *
