@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,11 +36,6 @@
 #include "rb-refstring.h"
 #include "rb-metadata.h"
 
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-async-ops.h>
-
 G_BEGIN_DECLS
 
 RhythmDBEntry * rhythmdb_entry_allocate		(RhythmDB *db, RhythmDBEntryType type);
@@ -153,17 +148,18 @@ struct _RhythmDBPrivate
 	GThreadPool *query_thread_pool;
 
 	GList *stat_list;
-	GHashTable *stat_events;
-	GnomeVFSAsyncHandle *stat_handle;
 	GList *outstanding_stats;
 	GMutex *stat_mutex;
+	gboolean stat_thread_running;
 
+	GVolumeMonitor *volume_monitor;
 	GHashTable *monitored_directories;
 	GHashTable *changed_files;
 	guint library_location_notify_id;
 	guint changed_files_id;
 	GSList *library_locations;
 	guint monitor_notify_id;
+	GMutex *monitor_mutex;
 
 	gboolean dry_run;
 	gboolean no_update;
@@ -176,7 +172,7 @@ struct _RhythmDBPrivate
 	GHashTable *propname_map;
 
 	GMutex *exit_mutex;
-	gboolean exiting;
+	GCancellable *exiting;		/* hrm, name? */
 
 	GCond *saving_condition;
 	GMutex *saving_mutex;
@@ -189,6 +185,7 @@ struct _RhythmDBPrivate
 	guint emit_entry_signals_id;
 	GList *added_entries_to_emit;
 	GList *deleted_entries_to_emit;
+	GHashTable *changed_entries_to_emit;
 
 	gboolean can_save;
 	gboolean saving;
@@ -224,8 +221,7 @@ typedef struct
 	RhythmDB *db;
 
 	/* STAT */
-	GnomeVFSFileInfo *vfsinfo;
-	GnomeVFSAsyncHandle *handle;
+	GFileInfo *file_info;
 	/* LOAD */
 	RBMetaData *metadata;
 	/* QUERY_COMPLETE */
@@ -238,7 +234,6 @@ typedef struct
 } RhythmDBEvent;
 
 /* from rhythmdb.c */
-void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type, RhythmDBEntryType ignore_type, RhythmDBEntryType error_type);
 void rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 				    gboolean visibility);
 void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
