@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,9 +35,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
-#include <gsequence.h>
+#include <glib.h>
 
-#include <gtk/gtktreednd.h>
+#include <gtk/gtk.h>
 
 #include "rhythmdb-query-model.h"
 #include "rb-debug.h"
@@ -262,6 +270,7 @@ enum
 	ENTRY_REMOVED,
 	NON_ENTRY_DROPPED,
 	POST_ENTRY_DELETE,
+	FILTER_ENTRY_DROP,
 	LAST_SIGNAL
 };
 
@@ -395,6 +404,25 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 			      g_cclosure_marshal_VOID__BOXED,
 			      G_TYPE_NONE,
 			      1, RHYTHMDB_TYPE_ENTRY);
+	/**
+	 * RhythmDBQueryModel::filter-entry-drop:
+	 * @model: the #RhythmDBQueryModel
+	 * @entry: the #RhythmDBEntry being dropped
+	 *
+	 * Emitted when an entry is being added to a model by drag-and-drop.
+	 * This allows the owner of the model to filter out entries that should
+	 * not be added to the model (based on entry type, for instance).
+	 * If the signal handler returns %FALSE, the entry will not be added.
+	 */
+	rhythmdb_query_model_signals[FILTER_ENTRY_DROP] =
+		g_signal_new ("filter-entry-drop",
+			      RHYTHMDB_TYPE_QUERY_MODEL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, filter_entry_drop),
+			      NULL, NULL,
+			      rb_marshal_BOOLEAN__BOXED,
+			      G_TYPE_BOOLEAN,
+			      1, RHYTHMDB_TYPE_ENTRY);
 
 	g_type_class_add_private (klass, sizeof (RhythmDBQueryModelPrivate));
 }
@@ -455,14 +483,12 @@ rhythmdb_query_model_set_query_internal (RhythmDBQueryModel *model,
 	rhythmdb_query_preprocess (model->priv->db, model->priv->query);
 
 	/* if the query contains time-relative criteria, re-run it periodically.
-	 * currently it's just every half hour, but perhaps it could be smarter.
+	 * currently it's just every minute, but perhaps it could be smarter.
 	 */
 	if (rhythmdb_query_is_time_relative (model->priv->db, model->priv->query)) {
 		if (model->priv->query_reapply_timeout_id == 0) {
 			model->priv->query_reapply_timeout_id =
-				g_timeout_add (1 * 60 * 1000,
-					       (GSourceFunc) rhythmdb_query_model_reapply_query_cb,
-					       model);
+				g_timeout_add_seconds (60, (GSourceFunc) rhythmdb_query_model_reapply_query_cb, model);
 		}
 	} else if (model->priv->query_reapply_timeout_id) {
 		g_source_remove (model->priv->query_reapply_timeout_id);
@@ -1881,8 +1907,20 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 
 				old_ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 				/* trying to drag drop an entry on itself ! */
-				if (old_ptr == ptr)
+				if (old_ptr == ptr) {
 					continue;
+				} else if (old_ptr == NULL) {
+					gboolean allow;
+
+					g_signal_emit (G_OBJECT (model),
+						       rhythmdb_query_model_signals[FILTER_ENTRY_DROP],
+						       0, entry, &allow);
+					if (allow == FALSE) {
+						rb_debug ("dropping of entry %s disallowed by filter",
+							  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+						continue;
+					}
+				}
 
 				/* take temporary ref */
 				rhythmdb_entry_ref (entry);
@@ -1929,14 +1967,13 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 	return FALSE;
 }
 
+/* 
+ * determines whether reordering is possible by checking up
+ * the chain for a model with a sort function set.
+ */
 static gboolean
-rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
-					GtkTreePath *dest,
-					GtkTreeViewDropPosition pos,
-					GtkSelectionData *selection_data)
+query_model_chain_can_reorder (RhythmDBQueryModel *model)
 {
-	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
-
 	while (model) {
 		if (model->priv->sort_func != NULL)
 			return FALSE;
@@ -1946,13 +1983,24 @@ rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
 	return TRUE;
 }
 
+static gboolean
+rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
+					GtkTreePath *dest,
+					GtkTreeViewDropPosition pos,
+					GtkSelectionData *selection_data)
+{
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
+	return query_model_chain_can_reorder (model);
+}
+
 static gboolean
 rhythmdb_query_model_row_drop_position (RbTreeDragDest *drag_dest,
 					GtkTreePath *dest_path,
 					GList *targets,
 					GtkTreeViewDropPosition *pos)
 {
-	return TRUE;
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (drag_dest);
+	return query_model_chain_can_reorder (model);
 }
 
 static void
