@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -56,7 +56,6 @@
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-app-helper.h>
-#include <libgnomeui/gnome-authentication-manager.h>
 
 #include <gst/gst.h>
 
@@ -167,10 +166,6 @@ main (int argc, char **argv)
 
 	rb_profile_end ("initializing gnome program");
 
-	rb_profile_start ("initializing gnome auth manager");
-	gnome_authentication_manager_init ();
-	rb_profile_end ("initializing gnome auth manager");
-
 	g_random_set_seed (time (0));
 
 #ifdef ENABLE_NLS
@@ -262,7 +257,11 @@ main (int argc, char **argv)
 
 		rb_refstring_system_init ();
 
-		rb_file_helpers_init ();
+#ifdef USE_UNINSTALLED_DIRS
+		rb_file_helpers_init (TRUE);
+#else
+		rb_file_helpers_init (FALSE);
+#endif
 
 		rb_debug ("Going to create a new shell");
 
@@ -272,6 +271,8 @@ main (int argc, char **argv)
 
 		gtk_window_set_default_icon_name ("rhythmbox");
 
+		g_setenv ("PULSE_PROP_media.role", "music", TRUE);
+
 		rb_shell = rb_shell_new (no_registration, no_update, dry_run, rhythmdb_file, playlists_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
 		if (!no_registration && session_bus != NULL) {
@@ -343,8 +344,6 @@ main (int argc, char **argv)
 		rb_file_helpers_shutdown ();
 		rb_stock_icons_shutdown ();
 		rb_refstring_system_shutdown ();
-
-		gnome_vfs_shutdown ();
 	}
 
 	gst_deinit ();
@@ -369,16 +368,24 @@ load_uri_args (const char **args, GFunc handler, gpointer user_data)
 
 	handled = FALSE;
 	for (i = 0; args && args[i]; i++) {
+		GFile *file;
 		char *uri;
 
 		rb_debug ("examining argument %s", args[i]);
 
-		uri = gnome_vfs_make_uri_from_shell_arg (args[i]);
+		file = g_file_new_for_commandline_arg (args[i]);
+		uri = g_file_get_uri (file);
 
+		/*
+		 * rb_uri_exists won't work if the location isn't mounted.
+		 * however, things that are interesting to mount are generally
+		 * non-local, so we'll process them anyway.
+		 */
 		if (rb_uri_is_local (uri) == FALSE || rb_uri_exists (uri)) {
 			handler (uri, user_data);
 		}
 		g_free (uri);
+		g_object_unref (file);
 
 		handled = TRUE;
 	}
