@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,9 +28,12 @@
  *
  */
 
-/* This class is a base case for weighted random play orders. Subclasses only
- * need to override get_entry_weight() to return the right weight for a given
- * entry.
+/**
+ * SECTION:rb-play-order-random
+ * @short_description: base class for weighted random play orders
+ *
+ * Subclasses only need to override get_entry_weight() to return the
+ * right weight for a given entry.
  *
  * This class also delays committing any changes until the user moves to the
  * next or previous song. So if the user changes the entry-view to contain
@@ -351,16 +362,11 @@ rb_random_play_order_go_next (RBPlayOrder* porder)
 	if (entry)
 		rhythmdb_entry_unref (entry);
 
-	g_object_get (rorder, "playing-entry", &entry, NULL);
-	g_assert (entry == NULL || rb_history_current (history) == NULL || entry == rb_history_current (history));
 	if (rb_history_current (history) == NULL)
 		rb_history_go_first (history);
 	else
 		rb_history_go_next (history);
 	rb_play_order_set_playing_entry (porder, rb_history_current (history));
-
-	if (entry)
-		rhythmdb_entry_unref (entry);
 }
 
 static RhythmDBEntry*
