@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,6 +29,25 @@
  *
  */
 
+/**
+ * SECTION:rb-shell-preferences
+ * @short_description: preferences dialog
+ *
+ * The preferences dialog is built around a #GtkNotebook widget, with two built-in
+ * pages and additional pages for various sources.
+ *
+ * The 'general' preferences page controls the set of browser views that are visible
+ * (artist and album; genre and artist; or genre, artist, and album), the columns
+ * that are visible, and the appearance of buttons in the main toolbar.  The browser
+ * and column settings apply to all sources.
+ *
+ * The 'playback' preferences page controls whether the crossfading player backend is used,
+ * and if enabled, the crossfade duration and network buffer size.
+ *
+ * Currently, the library and podcast sources add pages to the notebook, for configuring the
+ * location and layout of the library and the podcast download location and update frequency.
+ */
+
 #include <config.h>
 
 #include <string.h>
@@ -28,7 +55,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
+
+#if !GTK_CHECK_VERSION(2,13,1)
 #include <libgnome/gnome-help.h>
+#endif
 
 #include "rb-file-helpers.h"
 #include "rb-shell-preferences.h"
@@ -60,7 +90,6 @@ void rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 						      RBShellPreferences *shell_preferences);
 static void rb_shell_preferences_toolbar_style_cb (GtkComboBox *box,
 						   RBShellPreferences *preferences);
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 static void rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
 						    RBShellPreferences *preferences);
 static void rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
@@ -70,7 +99,6 @@ static void rb_shell_preferences_transition_duration_cb (GtkRange *range,
 static void rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
 							 RBShellPreferences *preferences);
 static void update_playback_prefs_sensitivity (RBShellPreferences *preferences);
-#endif
 
 enum
 {
@@ -95,13 +123,12 @@ struct RBShellPreferencesPrivate
 	GtkWidget *first_seen_check;
 	GtkWidget *quality_check;
 	GtkWidget *year_check;
+	GtkWidget *location_check;
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 	GtkWidget *xfade_backend_check;
 	GtkWidget *album_crossfade_check;
 	GtkWidget *transition_duration;
 	GtkWidget *network_buffer_size;
-#endif
 
 	GSList *browser_views_group;
 
@@ -130,7 +157,14 @@ help_cb (GtkWidget *widget,
 {
 	GError *error = NULL;
 
+#if GTK_CHECK_VERSION(2,13,1)
+	gtk_show_uri (gtk_widget_get_screen (widget),
+		      "ghelp:rhythmbox",
+		      gtk_get_current_event_time (),
+		      &error);
+#else
 	gnome_help_display ("rhythmbox.xml", "prefs", &error);
+#endif
 
 	if (error != NULL) {
 		rb_error_dialog (NULL,
@@ -211,6 +245,8 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 		glade_xml_get_widget (xml, "year_check");
 	shell_preferences->priv->first_seen_check =
 		glade_xml_get_widget (xml, "first_seen_check");
+	shell_preferences->priv->location_check =
+		glade_xml_get_widget (xml, "location_check");
 
 	/* browser options */
 	rb_glade_boldify_label (xml, "browser_views_label");
@@ -240,7 +276,6 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				    shell_preferences);
 	g_object_unref (xml);
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 	/* playback preferences */
 	xml = rb_glade_xml_new ("playback-prefs.glade",
 				"playback_prefs_box",
@@ -284,7 +319,6 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				  glade_xml_get_widget (xml, "playback_prefs_box"),
 				  gtk_label_new (_("Playback")));
 	g_object_unref (xml);
-#endif
 
 	eel_gconf_notification_add (CONF_PLAYER_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
@@ -308,6 +342,14 @@ rb_shell_preferences_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_shell_preferences_parent_class)->finalize (object);
 }
 
+/**
+ * rb_shell_preferences_append_page:
+ * @prefs: the #RBShellPreferences instance
+ * @name: name of the page to append
+ * @widget: the #GtkWidget to use as the contents of the page
+ *
+ * Appends a new page to the preferences dialog notebook.
+ */
 void
 rb_shell_preferences_append_page (RBShellPreferences *prefs,
 				  const char *name,
@@ -338,6 +380,15 @@ rb_shell_preferences_append_view_page (RBShellPreferences *prefs,
 	rb_shell_preferences_append_page (prefs, name, widget);
 }
 
+/**
+ * rb_shell_preferences_new:
+ * @views: list of #RBSource objects to check for preferences pages
+ *
+ * Creates the #RBShellPreferences instance, populating it with the
+ * preferences pages for the sources in the list.
+ *
+ * Return value: the #RBShellPreferences instance
+ */
 GtkWidget *
 rb_shell_preferences_new (GList *views)
 {
@@ -398,6 +449,14 @@ rb_shell_preferences_ui_pref_changed (GConfClient *client,
 	rb_shell_preferences_sync (shell_preferences);
 }
 
+/**
+ * rb_shell_preferences_column_check_changed_cb:
+ * @butt: the #GtkCheckButton that was changed
+ * @shell_preferences: the #RBShellPreferences instance
+ *
+ * Signal handler used for the checkboxes used to configure the set of visible columns.
+ * This updates the GConf key that contains the list of visible columns.
+ */
 void
 rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 					      RBShellPreferences *shell_preferences)
@@ -430,6 +489,8 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_BITRATE";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->first_seen_check))
 		colname = "RHYTHMDB_PROP_FIRST_SEEN";
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->location_check))
+		colname = "RHYTHMDB_PROP_LOCATION";
 	else
 		g_assert_not_reached ();
 
@@ -478,6 +539,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 	char *columns;
 	GSList  *l;
 	gint view, i;
+	gboolean b;
+	float duration;
+	int buffer_size;
 
 	shell_preferences->priv->loading = TRUE;
 
@@ -519,6 +583,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->quality_check,
 							 columns, "RHYTHMDB_PROP_BITRATE");
+		rb_shell_preferences_sync_column_button (shell_preferences,
+			       				 shell_preferences->priv->location_check,
+							 columns, "RHYTHMDB_PROP_LOCATION");
 	}
 
 	g_free (columns);
@@ -542,28 +609,20 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 					   G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
 					   shell_preferences);
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
-	{
-		gboolean b;
-		float duration;
-		int buffer_size;
+	/* player preferences */
+	b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
 
-		/* player preferences */
-		b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
+	b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
 
-		b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
+	duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
+	gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
 
-		duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
-		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
+	buffer_size = eel_gconf_get_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE);
+	gtk_range_set_value (GTK_RANGE (shell_preferences->priv->network_buffer_size), buffer_size);
 
-		buffer_size = eel_gconf_get_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE);
-		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->network_buffer_size), buffer_size);
-
-		update_playback_prefs_sensitivity (shell_preferences);
-	}
-#endif
+	update_playback_prefs_sensitivity (shell_preferences);
 
 	shell_preferences->priv->loading = FALSE;
 }
@@ -582,6 +641,14 @@ rb_shell_preferences_toolbar_style_cb (GtkComboBox *box, RBShellPreferences *pre
 	eel_gconf_set_integer (CONF_UI_TOOLBAR_STYLE, selection);
 }
 
+/**
+ * rb_shell_preferences_browser_views_activated_cb:
+ * @widget: the radio button that was selected
+ * @shell_preferences: the #RBShellPreferences instance
+ *
+ * Signal handler used for the radio buttons used to configure the
+ * visible browser views.
+ */
 void
 rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 						 RBShellPreferences *shell_preferences)
@@ -596,8 +663,6 @@ rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 	eel_gconf_set_integer (CONF_UI_BROWSER_VIEWS, index);
 }
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
-
 static void
 update_playback_prefs_sensitivity (RBShellPreferences *preferences)
 {
@@ -651,4 +716,3 @@ rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
 	eel_gconf_set_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE, (int)v);
 }
 
-#endif
