@@ -7,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,15 +44,18 @@ G_BEGIN_DECLS
 #define RB_IS_AUTO_PLAYLIST_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_AUTO_PLAYLIST_SOURCE))
 #define RB_AUTO_PLAYLIST_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_AUTO_PLAYLIST_SOURCE, RBAutoPlaylistSourceClass))
 
-typedef struct
+typedef struct _RBAutoPlaylistSource RBAutoPlaylistSource;
+typedef struct _RBAutoPlaylistSourceClass RBAutoPlaylistSourceClass;
+
+struct _RBAutoPlaylistSource
 {
 	RBPlaylistSource parent;
-} RBAutoPlaylistSource;
+};
 
-typedef struct
+struct _RBAutoPlaylistSourceClass
 {
 	RBPlaylistSourceClass parent;
-} RBAutoPlaylistSourceClass;
+};
 
 GType		rb_auto_playlist_source_get_type 	(void);
 
