@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -72,7 +72,7 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib.h>
 
 #include "rb-tree-dnd.h"
 #include "rb-entry-view.h"
@@ -521,17 +521,17 @@ rb_entry_view_init (RBEntryView *view)
 	icon_theme = gtk_icon_theme_get_default ();
 
 	view->priv->playing_pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                   			       "stock_media-play",
+                                   			       "media-playback-start",
                                    			       16,
                                    			       0,
                                    			       NULL);
 	view->priv->paused_pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                   			      "stock_media-pause",
+                                   			      "media-playback-pause",
                                    			      16,
                                    			      0,
                                    			      NULL);
 	view->priv->error_pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                   			     "stock_dialog-error",
+                                   			     "dialog-error",
                                    			     16,
                                    			     0,
                                    			     NULL);
@@ -589,6 +589,12 @@ rb_entry_view_dispose (GObject *object)
 	}
 
 	if (view->priv->model != NULL) {
+		/* remove the model from the treeview so
+		 * atk-bridge doesn't have to emit deletion events
+		 * for each cell in the view.
+		 */
+		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview), NULL);
+
 		g_object_unref (view->priv->model);
 		view->priv->model = NULL;
 	}
@@ -1077,7 +1083,7 @@ rb_entry_view_location_cell_data_func (GtkTreeViewColumn *column,
 	entry = rhythmdb_query_model_iter_to_entry (data->view->priv->model, iter);
 
 	location = rhythmdb_entry_get_string (entry, data->propid);
-	str = gnome_vfs_unescape_string_for_display (location);
+	str = g_uri_unescape_string (location, NULL);
 
 	g_object_set (renderer, "text", str, NULL);
 	g_free (str);
@@ -1786,13 +1792,10 @@ rb_entry_view_constructor (GType type,
 
 	{
 		GtkTreeViewColumn *column;
-		GtkTooltips *tooltip;
 		GtkCellRenderer *renderer;
 		GtkWidget *image_widget;
 		gint width;
 
-		tooltip = gtk_tooltips_new ();
-
 		/* Playing icon column */
 		column = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
 		renderer = rb_cell_renderer_pixbuf_new ();
@@ -1803,7 +1806,7 @@ rb_entry_view_constructor (GType type,
 							 view,
 							 NULL);
 
-		image_widget = gtk_image_new_from_icon_name ("stock_volume-max", GTK_ICON_SIZE_MENU);
+		image_widget = gtk_image_new_from_icon_name ("audio-volume-high", GTK_ICON_SIZE_MENU);
 		gtk_tree_view_column_set_widget (column, image_widget);
 		gtk_widget_show (image_widget);
 
@@ -1817,8 +1820,8 @@ rb_entry_view_constructor (GType type,
 					  G_CALLBACK (rb_entry_view_pixbuf_clicked_cb),
 					  view);
 
-		gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltip), GTK_WIDGET (column->button),
-				       _("Now Playing"), NULL);
+		gtk_widget_set_tooltip_text (GTK_WIDGET (column->button),
+					     _("Now Playing"));
 	}
 
 	view->priv->gconf_notification_id =
@@ -2542,7 +2545,7 @@ rb_entry_view_resort_model (RBEntryView *view)
 	rhythmdb_query_model_set_sort_order (view->priv->model,
 					     sort_data->func,
 					     sort_data->data,
-					     sort_data->data_destroy,
+					     NULL,
 					     (view->priv->sorting_order == GTK_SORT_DESCENDING));
 }
 
