@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,9 +30,9 @@
 
 #include "config.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <glib.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
 #include "rb-query-creator-private.h"
@@ -275,7 +283,7 @@ stringCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 static void
 escapedStringCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 {
-	char *text = gnome_vfs_unescape_string (g_value_get_string (val), NULL);
+	char *text = g_uri_unescape_string (g_value_get_string (val), NULL);
 	gtk_entry_set_text (GTK_ENTRY (widget), text);
 	g_free (text);
 }
@@ -283,7 +291,7 @@ escapedStringCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 static void
 escapedStringCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 {
-	char *text = gnome_vfs_escape_host_and_path_string (gtk_entry_get_text (GTK_ENTRY (widget)));
+	char *text = g_uri_escape_string (gtk_entry_get_text (GTK_ENTRY (widget)), G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
 
 	g_value_init (val, G_TYPE_STRING);
 	g_value_set_string (val, text);
@@ -418,7 +426,7 @@ durationCriteriaCreateWidget (gboolean *constrain)
 	/* the widget for Duration is set out like the following [ 2] : [30] */
 	box = GTK_BOX (gtk_hbox_new (FALSE, 3));
 
-	minutesSpin = gtk_spin_button_new_with_range (0.0, G_MAXINT, 1.0);
+	minutesSpin = gtk_spin_button_new_with_range (0.0, (double)((G_MAXINT - 59) / 60), 1.0);
 	gtk_box_pack_start (box, minutesSpin, FALSE, FALSE, 0);
 
 	minutesLabel = gtk_label_new (":");
@@ -460,23 +468,29 @@ durationCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
  * Implementation for the relative time properties, using a spin button and a menu.
  */
 
+static void
+update_time_unit_limits (GtkComboBox *menu, GtkWidget *spin_button)
+{
+	/* set the range on the spin button so it can't overflow when
+	 * converted to seconds when we're constructing the query
+	 */
+	gulong mult = time_unit_options [gtk_combo_box_get_active (menu)].timeMultiplier;
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (spin_button), 1, G_MAXINT / mult);
+}
+
 static GtkWidget*
 create_time_unit_option_menu (const RBQueryCreatorTimeUnitOption *options,
 			     int length)
 {
-	GtkWidget *menu = gtk_menu_new ();
-	GtkWidget *option_menu = gtk_option_menu_new ();
+	GtkWidget *combo;
 	int i;
 
+	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		GtkWidget *menu_item = gtk_menu_item_new_with_label (_(options[i].name));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
 	}
 
-	gtk_widget_show_all (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-
-	return option_menu;
+	return combo;
 }
 
 static GtkWidget *
@@ -490,11 +504,15 @@ relativeTimeCriteriaCreateWidget (gboolean *constrain)
 	box = GTK_BOX (gtk_hbox_new (FALSE, 6));
 
 	timeSpin = gtk_spin_button_new_with_range (1.0, G_MAXINT, 1.0);
-	gtk_box_pack_start_defaults (box, timeSpin);
+	gtk_box_pack_start (box, timeSpin, TRUE, TRUE, 0);
 
 	timeOption = create_time_unit_option_menu (time_unit_options, G_N_ELEMENTS (time_unit_options));
-	gtk_option_menu_set_history(GTK_OPTION_MENU (timeOption), time_unit_options_default);
-	gtk_box_pack_start_defaults (box, timeOption);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (timeOption), time_unit_options_default);
+	gtk_box_pack_start (box, timeOption, TRUE, TRUE, 0);
+	
+	g_signal_connect_object (timeOption, "changed",
+				 G_CALLBACK (update_time_unit_limits),
+				 timeSpin, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (box));
 	return GTK_WIDGET (box);
@@ -506,7 +524,7 @@ relativeTimeCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 	GtkBox *box = GTK_BOX (widget);
 
 	GtkSpinButton *timeSpin = GTK_SPIN_BUTTON (get_box_widget_at_pos (box, 0));
-	GtkOptionMenu *unitMenu = GTK_OPTION_MENU (get_box_widget_at_pos (box, 1));
+	GtkComboBox *unitMenu = GTK_COMBO_BOX (get_box_widget_at_pos (box, 1));
 
 	gulong time = g_value_get_ulong (val);
 	gulong unit = 0;
@@ -522,17 +540,17 @@ relativeTimeCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 	time = time / time_unit_options[unit].timeMultiplier;
 	g_assert (time < G_MAXINT);
 	/* set the time value and unit*/
-	gtk_option_menu_set_history(unitMenu, unit);
-	gtk_spin_button_set_value(timeSpin, time);
+	gtk_combo_box_set_active (unitMenu, unit);
+	gtk_spin_button_set_value (timeSpin, time);
 }
 
 static void
 relativeTimeCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 {
 	GtkSpinButton *timeSpin = GTK_SPIN_BUTTON (get_box_widget_at_pos (GTK_BOX (widget), 0));
-	GtkOptionMenu *unitMenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (widget), 1));
+	GtkComboBox *unitMenu = GTK_COMBO_BOX (get_box_widget_at_pos (GTK_BOX (widget), 1));
 
-	gulong timeMultiplier = time_unit_options [gtk_option_menu_get_history (unitMenu)].timeMultiplier;
+	gulong timeMultiplier = time_unit_options [gtk_combo_box_get_active (unitMenu)].timeMultiplier;
 	gint value = gtk_spin_button_get_value_as_int (timeSpin) * timeMultiplier;
 	g_assert (value >= 0);
 
