@@ -2,7 +2,7 @@
  *
  *  arch-tag: Implementation of local song properties dialog
  *
- *  Copyright (C) 2002 Olivier Martin <omartin@ifrance.com>
+ *  Copyright (C) 2002 Olivier Martin <olive.martin@gmail.com>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -10,6 +10,15 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,8 +45,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
 #include "rhythmdb-property-model.h"
@@ -49,6 +56,7 @@
 #include "eel-gconf-extensions.h"
 #include "rb-source.h"
 #include "rb-shell.h"
+#include "rb-file-helpers.h"
 
 static void rb_song_info_class_init (RBSongInfoClass *klass);
 static void rb_song_info_init (RBSongInfo *song_info);
@@ -90,6 +98,9 @@ static void rb_song_info_forward_clicked_cb (GtkWidget *button,
 static void rb_song_info_query_model_changed_cb (GObject *source,
 						 GParamSpec *pspec,
 						 RBSongInfo *song_info);
+static void rb_song_info_base_query_model_changed_cb (GObject *source,
+						      GParamSpec *pspec,
+						      RBSongInfo *song_info);
 static void rb_song_info_rated_cb (RBRating *rating,
 				   double score,
 				   RBSongInfo *song_info);
@@ -102,6 +113,7 @@ struct RBSongInfoPrivate
 	RBSource *source;
 	RBEntryView *entry_view;
 	RhythmDBQueryModel *query_model;
+	RhythmDBQueryModel *base_query_model;
 
 	/* information on the displayed song */
 	RhythmDBEntry *current_entry;
@@ -141,6 +153,35 @@ struct RBSongInfoPrivate
 
 #define RB_SONG_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SONG_INFO, RBSongInfoPrivate))
 
+/**
+ * SECTION:rb-song-info
+ * @short_description: song properties dialog
+ *
+ * Displays song properties and, if we know how to edit tags in the file,
+ * allows the user to edit them.
+ *
+ * This class has two modes.  It can display and edit properties of a single
+ * entry, in which case it uses a #GtkNotebook to split the properties across
+ * 'basic' and 'details' pages, and it can display and edit properties of
+ * multiple entries at a time, in which case a smaller set of properties is
+ * displayed in a single set.
+ *
+ * In single-entry mode, it is possible to add extra pages to the #GtkNotebook
+ * widget in the dialog.  The 'create-song-info' signal is emitted by the #RBShell
+ * object, allowing signal handlers to add pages by calling #rb_song_info_append_page.
+ * The lyrics plugin is currently the only place where this ability is used.
+ * In this mode, the dialog features 'back' and 'forward' buttons that move to the
+ * next or previous entries from the currently displayed track list.
+ *
+ * In multiple-entry mode, only the set of properties that can usefully be set
+ * across multiple entries at once are displayed.
+ *
+ * When the dialog is closed, any changes made will be applied to the entry (or entries)
+ * that were displayed in the dialog.  For songs in the library, this will result
+ * in the song tags being updated on disk.  For other entry types, this only updates
+ * the data store in the database.
+ */
+
 enum
 {
 	PRE_METADATA_CHANGE,
@@ -173,6 +214,13 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 
 	widget_class->show = rb_song_info_show;
 
+	/**
+	 * RBSongInfo:source:
+	 *
+	 * The #RBSource that created the song properties window.  Used to update
+	 * for track list changes, and to find the sets of albums, artist, and genres
+	 * to use for tag edit completion.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SOURCE,
 					 g_param_spec_object ("source",
@@ -180,6 +228,13 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      "RBSource object",
 					                      RB_TYPE_SOURCE,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBSongInfo:entry-view:
+	 *
+	 * The #RBEntryView for the source that created the song properties window.  Used
+	 * find the set of selected entries, and to change the selection when the 'back' and
+	 * 'forward' buttons are pressed.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_VIEW,
 					 g_param_spec_object ("entry-view",
@@ -187,6 +242,12 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      "RBEntryView object",
 					                      RB_TYPE_ENTRY_VIEW,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBSongInfo:current-entry:
+	 *
+	 * The #RhythmDBEntry that is currently being displayed.  Will be NULL for
+	 * multiple-entry song properties windows.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_CURRENT_ENTRY,
 					 g_param_spec_boxed ("current-entry",
@@ -195,6 +256,12 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 							     RHYTHMDB_TYPE_ENTRY,
 					                     G_PARAM_READABLE));
 
+	/**
+	 * RBSongInfo:selected-entries:
+	 *
+	 * The set of #RhythmDBEntry objects currently being displayed.  Valid for both
+	 * single-entry and multiple-entry song properties windows.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SELECTED_ENTRIES,
 					 g_param_spec_value_array ("selected-entries",
@@ -210,6 +277,15 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 	object_class->dispose = rb_song_info_dispose;
 	object_class->finalize = rb_song_info_finalize;
 
+	/**
+	 * RBSongInfo::pre-metadata-change:
+	 * @song_info: the #RBSongInfo instance
+	 * @entry: the #RhythmDBEntry being changed
+	 *
+	 * Emitted just before the changes made in the song properties window
+	 * are applied to the database.  This is only emitted in the single-entry
+	 * case.
+	 */
 	rb_song_info_signals[PRE_METADATA_CHANGE] =
 		g_signal_new ("pre-metadata-change",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -221,6 +297,14 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 			      1,
 			      RHYTHMDB_TYPE_ENTRY);
 
+	/**
+	 * RBSongInfo::post-metadata-change:
+	 * @song_info: the #RBSongInfo instance
+	 * @entry: the #RhythmDBEntry that was changed
+	 *
+	 * Emitted just after changes have been applied to the database.
+	 * Probably useless.
+	 */
 	rb_song_info_signals[POST_METADATA_CHANGE] =
 		g_signal_new ("post-metadata-change",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -362,6 +446,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	GList *tem;
 	gboolean editable = TRUE;
 	RBShell *shell;
+	AtkObject *lobj, *robj;
 
 	klass = RB_SONG_INFO_CLASS (g_type_class_peek (RB_TYPE_SONG_INFO));
 
@@ -457,6 +542,13 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "song_info_rating_container")),
 			   song_info->priv->rating);
 
+	/* add relationship between the rating label and the rating widget */
+	lobj = gtk_widget_get_accessible (glade_xml_get_widget (xml, "rating_label"));
+	robj = gtk_widget_get_accessible (song_info->priv->rating);
+
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
+
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->artist), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->album), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->genre), editable);
@@ -519,6 +611,9 @@ rb_song_info_dispose (GObject *object)
 		g_signal_handlers_disconnect_by_func (song_info->priv->source,
 						      G_CALLBACK (rb_song_info_query_model_changed_cb),
 						      song_info);
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      G_CALLBACK (rb_song_info_base_query_model_changed_cb),
+						      song_info);
 		g_object_unref (song_info->priv->source);
 		song_info->priv->source = NULL;
 	}
@@ -554,12 +649,13 @@ static void
 rb_song_info_set_source_internal (RBSongInfo *song_info,
 				  RBSource   *source)
 {
-	RhythmDB *old_db = song_info->priv->db;
-
 	if (song_info->priv->source != NULL) {
 		g_signal_handlers_disconnect_by_func (song_info->priv->source,
 						      rb_song_info_query_model_changed_cb,
 						      song_info);
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      rb_song_info_base_query_model_changed_cb,
+						      song_info);
 		g_object_unref (song_info->priv->source);
 		g_object_unref (song_info->priv->query_model);
 		g_object_unref (song_info->priv->db);
@@ -575,43 +671,14 @@ rb_song_info_set_source_internal (RBSongInfo *song_info,
 				 "notify::query-model",
 				 G_CALLBACK (rb_song_info_query_model_changed_cb),
 				 song_info, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->source),
+				 "notify::base-query-model",
+				 G_CALLBACK (rb_song_info_base_query_model_changed_cb),
+				 song_info, 0);
 
 	g_object_get (G_OBJECT (song_info->priv->query_model), "db", &song_info->priv->db, NULL);
 
-	if (old_db != song_info->priv->db) {
-		if (song_info->priv->albums) {
-			g_object_unref (song_info->priv->albums);
-		}
-		if (song_info->priv->artists) {
-			g_object_unref (song_info->priv->artists);
-		}
-		if (song_info->priv->genres) {
-			g_object_unref (song_info->priv->genres);
-		}
-
-		song_info->priv->albums  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ALBUM);
-		song_info->priv->artists = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ARTIST);
-		song_info->priv->genres  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_GENRE);
-
-		g_object_set (song_info->priv->albums,  "query-model", song_info->priv->query_model, NULL);
-		g_object_set (song_info->priv->artists, "query-model", song_info->priv->query_model, NULL);
-		g_object_set (song_info->priv->genres,  "query-model", song_info->priv->query_model, NULL);
-
-		if(song_info->priv->album) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
-		}
-
-		if(song_info->priv->artist) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
-		}
-
-		if(song_info->priv->genre) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
-		}
-	}
+	rb_song_info_base_query_model_changed_cb (G_OBJECT (song_info->priv->source), NULL, song_info);
 }
 
 static void
@@ -677,6 +744,16 @@ rb_song_info_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_song_info_new:
+ * @source: #RBSource creating the song properties window
+ * @entry_view: the #RBEntryView to get selection data from
+ *
+ * Creates a new #RBSongInfo for the selected entry or entries in
+ * the specified entry view.
+ *
+ * Return value: the new song properties window
+ */
 GtkWidget *
 rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 {
@@ -700,6 +777,17 @@ rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 	return GTK_WIDGET (song_info);
 }
 
+/**
+ * rb_song_info_append_page:
+ * @info: a #RBSongInfo
+ * @title: the title of the new page
+ * @page: the page #GtkWidget
+ *
+ * Adds a new page to the song properties window.  Should be called
+ * in a handler connected to the #RBShell 'create-song-info' signal.
+ *
+ * Return value: the page number
+ */
 guint
 rb_song_info_append_page (RBSongInfo *info, const char *title, GtkWidget *page)
 {
@@ -989,7 +1077,7 @@ rb_song_info_update_filesize (RBSongInfo *song_info)
 	char *text = NULL;
 	guint64 filesize = 0;
 	filesize = rhythmdb_entry_get_uint64 (song_info->priv->current_entry, RHYTHMDB_PROP_FILE_SIZE);
-	text = gnome_vfs_format_file_size_for_display (filesize);
+	text = g_format_size_for_display (filesize);
 	gtk_label_set_text (GTK_LABEL (song_info->priv->filesize), text);
 	g_free (text);
 }
@@ -1008,10 +1096,12 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		char *basename, *dir, *desktopdir;
 
 		basename = g_path_get_basename (text);
-		tmp = gnome_vfs_unescape_string_for_display (basename);
+		tmp = g_uri_unescape_string (basename, NULL);
 		g_free (basename);
 		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
 		g_free (tmp);
+		tmp = NULL;
+
 		if (tmp_utf8 != NULL) {
 			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name),
 					    tmp_utf8);
@@ -1022,19 +1112,22 @@ rb_song_info_update_location (RBSongInfo *song_info)
 
 		g_free (tmp_utf8);
 
-		tmp = gnome_vfs_get_local_path_from_uri (text);
-		if (tmp == NULL)
+		if (rb_uri_is_local (text)) {
+			tmp = g_filename_from_uri (text, NULL, NULL);
+		}
+		if (tmp == NULL) {
 			tmp = g_strdup (text);
+		}
+
 		dir = g_path_get_dirname (tmp);
 		g_free (tmp);
-		tmp = gnome_vfs_unescape_string_for_display (dir);
+		tmp = g_uri_unescape_string (dir, NULL);
 		g_free (dir);
 		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
 		g_free (tmp);
 
 		desktopdir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
-		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0))
-		{
+		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0)) {
 			g_free (tmp_utf8);
 			tmp_utf8 = g_strdup (_("On the desktop"));
 		}
@@ -1150,6 +1243,51 @@ rb_song_info_query_model_reordered_cb (RhythmDBQueryModel *model,
 	rb_song_info_update_buttons (song_info);
 }
 
+static void
+rb_song_info_base_query_model_changed_cb (GObject *source,
+					  GParamSpec *whatever,
+					  RBSongInfo *song_info)
+{
+	RhythmDBQueryModel *base_query_model;
+
+	g_object_get (source, "base-query-model", &base_query_model, NULL);
+
+	if (song_info->priv->albums) {
+		g_object_unref (song_info->priv->albums);
+	}
+	if (song_info->priv->artists) {
+		g_object_unref (song_info->priv->artists);
+	}
+	if (song_info->priv->genres) {
+		g_object_unref (song_info->priv->genres);
+	}
+
+	song_info->priv->albums  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ALBUM);
+	song_info->priv->artists = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ARTIST);
+	song_info->priv->genres  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_GENRE);
+
+	g_object_set (song_info->priv->albums,  "query-model", base_query_model, NULL);
+	g_object_set (song_info->priv->artists, "query-model", base_query_model, NULL);
+	g_object_set (song_info->priv->genres,  "query-model", base_query_model, NULL);
+
+	if (song_info->priv->album) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
+	}
+
+	if (song_info->priv->artist) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
+	}
+
+	if (song_info->priv->genre) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
+	}
+
+	g_object_unref (base_query_model);
+}
+
 static void
 rb_song_info_query_model_changed_cb (GObject *source,
 				     GParamSpec *whatever,
